//  complejo.cpp: definiciones para el ejercicio 30
//  Fecha: 7:53 jueves, 25 de abril de 2019
//  Revisin: 24/9/2020; 5:44 pm
//  Por S. Galliano.

#include "complejo.h"
#include <iomanip>
#include <iostream>
#include <cmath>
#include <ios>

using std::cout;
using std::cin;

// constructor
Complejo::Complejo( double a, double b ) {
    real = a;
    imag = b;
}

// comprueba si dos complejos son iguales
bool Complejo::esIgualQue ( Complejo z ) const {
    return real == z.real and imag == z.imag; // a1 == a2 y b1 == b2
}

// suma dos complejos
Complejo Complejo::sumadoCon( Complejo z ) const {
    double a = real + z.real; // a = r + rz
    double b = imag + z.imag; // b = i + iz
    return Complejo( a, b );
}

// resta un complejo de otro
Complejo Complejo::restadoDe( Complejo z ) const {
    double a = real - z.real; // a = r - rz
    double b = imag - z.imag; // b = i - iz
    return Complejo( a, b );
}

// multiplica un complejo con otro
Complejo Complejo::multipCon( Complejo z ) const {
    double a = real * z.real - imag * z.imag;
    double b = real * z.imag + imag * z.real;
    return Complejo( a, b );
}

// divide un complejo con otro
Complejo Complejo::divideCon( Complejo z ) const {
    double cociente = pow( z.real, 2.0 ) + pow( z.imag, 2.0 );
    double F1 = ( real * z.real + imag * z.imag );
    double F2 = ( imag * z.real + real * z.imag );
    double a = F1 / cociente;
    double b = F2 / cociente;
    return Complejo( a, b );
}

// conjugado
Complejo Complejo::conjugado() const {
    return Complejo( real, -imag );
}

// mdulo
double Complejo::modulo() {
    return sqrt( pow( real, 2.0 ) + pow( imag, 2.0 ) );
}

// entra un nmero complejo
void Complejo::entraNumero( Str id ) {
    cout << id;
    cin.ignore( 1, '(' ) >> real; // ignora '('
    cin.ignore( 1, ',' ) >> imag; // ignora ','

    // en el bfer de entrada quedan restos: eliminar hasta
    // el ltimo carcter contenido en el bfer
    while ( cin.get() not_eq '\n' ) {
        continue;
    } // while
}

// muestra un nmero complejo en formato coordenado
void Complejo::fmtCoord( Str id ) const {
    std::ios::fmtflags oldFlags = cout.flags();
    cout << id << std::setprecision( 2 ) << std::fixed
         << "(" << real << "," << imag << ")\n"
         << std::setprecision( 6 );
    cout.flags( oldFlags );
}

// muestra un nmero complejo en formato de suma
void Complejo::fmtSuma( Str id ) {
    std::ios::fmtflags oldFlags = cout.flags();
    cout << id << std::setprecision( 2 ) << std::fixed << real
         << std::showpos << imag << "i\n" << std::setprecision( 6 );
    cout.flags( oldFlags );
}

// muestra un nmero complejo en formato polar
void Complejo::fmtPolar( Str id ) {
    double sita = atan2( imag, real ) * RAD2GRD;
    double modulo = sqrt( pow( real, 2.0 ) + pow( imag, 2.0 ) );
    std::ios::fmtflags oldFlags = cout.flags();
    cout << id << std::setprecision( 2 ) << std::fixed
         << "(" << modulo << "," << sita << ")\n"
         << std::setprecision( 6 );
    cout.flags( oldFlags );
}

// suma con un escalar
Complejo Complejo::sumadoCon( double e ) {
    double a = real + e;
    double b = imag;
    return Complejo( a, b );
}

// resta de un escalar
Complejo Complejo::restadoDe( double e ) {
    double a = abs( real - e );
    double b = imag;
    return Complejo( a, b );
}

// multiplicado con un escalar
Complejo Complejo::multipCon( double e ) {
    double a = real * e;
    double b = imag * e;
    return Complejo( a, b );
}

// dividido entre un escalar
Complejo Complejo::divideCon( double e ) {
    double a = real / e;
    double b = imag / e;
    return Complejo( a, b );
}
