//  biblio.cpp: definiciones para el ejercicio 37, sptimo bloque
//  Fecha: 14/8/2019; 10 am
//  Revisin: 6/9/2020; 6:26 am
//  Por S. Galliano.

#include "biblio.h"
#include <iostream>
#include <cstdlib>
#include <iomanip>
using std::cout;
using std::cin;
using std::getline;

//  funcin auxiliar para poner una cadena en maysculas
void toUpper( Str &ID ) {
    for ( unsigned i = 0; i < ID.size(); ++i ) {
        ID[i] = toupper( ID[i] );
    } // for
}

//  Obra
void Obra::entrarDato() {
    cout << "Entrando una obra (nombre, apellido y t\241tulo)\n";
    cout << "Autor (nombre): ";
    getline( cin, nombre );
    cout << "Autor (apellido): ";
    getline( cin, apellido );
    cout << "Obra (t\241tulo): ";
    getline( cin, titulo );
    id  = apellido[0];
    id += nombre[0];
    id += titulo[0];
    toUpper( id );
}

void Obra::mostrarDato() {
    cout << std::left << id << ' ' << std::setfill( '.' ) <<
    std::setw( 25 ) << ( apellido + ", " + nombre ) <<
    std::setfill( ' ' ) << std::right << titulo << '\n';
}

//   Biblio
Biblio::Biblio() {
    head = prox = nullptr;
    n = 0;
}

bool Biblio::estaVacia() const {
    return nullptr == head;
}

int Biblio::contarObras() const {
    return n;
}

bool Biblio::estaEnLista( Str val ) const {
    if ( estaVacia() ) {
        return false;
    } // if

    for ( Biblio * p1 = head; p1 not_eq nullptr; p1 = p1->prox ) {
        if ( val == p1->obra.id ) {
            return true;
        } // if
    } // for

    return false;
}

void Biblio::borrarObra( Str ID ) {
    if ( estaVacia() or not estaEnLista( ID ) ) {
        return;
    } // if

    Biblio *p1 = head, *p2 = nullptr;

    while ( p1->obra.id not_eq ID ) {
        p2 = p1;
        p1 = p1->prox;
    } // while

    if ( nullptr == p2 ) {
        head = p1->prox;
    } else {
        p2->prox = p1->prox;
    } // if-else

    --n;
    delete p1;
}

void Biblio::vaciarBiblio() {
    Biblio * p1;
    n = 0;

    while ( not estaVacia() ) {
        if ( head->prox == nullptr ) {
            delete head;
            head = nullptr;
        } else {
            p1 = head;
            head = p1->prox;
            delete p1;
        } // if-else
    } // while
}

void Biblio::entrarObra() { // en la cabeza
    Biblio *p1 = head, *p2 = nullptr;
    Biblio *nueva = new Biblio;
    nueva->obra.entrarDato();

    while ( p1 not_eq nullptr ) {
        if ( nueva->obra.id < p1->obra.id ) {
            nueva->prox = p1;       // inserta el nodo entre p2 y p1
            ++n;

            if ( nullptr == p2 ) {  // chequea que p2 no sea nulo
                head = nueva;
            } else {
                p2->prox = nueva;
            } // if-else

            return; // ya est: nos vamos
        } else {    // avanza
            p2 = p1;
            p1 = p1->prox;
        }
    } // while

    nueva->prox = nullptr; // no insert?
    ++n;                   // entonces, al final de la lista

    if ( nullptr == p2 ) { // chequea otra vez que p2 no sea nulo
        head = nueva;
    } else {
        p2->prox = nueva;  // aade
    } // if-else
}

void Biblio::mostrarBiblio() const {
    if ( estaVacia() ) {
        return;
    } // if

    for ( Biblio *p1 = head; p1 not_eq nullptr; p1 = p1->prox ) {
        p1->obra.mostrarDato();
    } // for

    cout << '\n';
}

void Biblio::mostrarObras( Str ID ) const {
    if ( estaVacia() ) {
        return;
    } // if

    for ( Biblio *p1 = head; p1 not_eq nullptr; p1 = p1->prox ) {
        if ( ID == p1->obra.id ) {
            p1->obra.mostrarDato();
            cout << '\n';
            return;
        } // if
    } // for
}
