//  nomina.cpp: definiciones para el ejercicio 40, noveno bloque
//  Fecha: 16/8/2019; 12:20 pm
//  Revisin: 6/9/2020; 8:30 am
//  Por S. Galliano.

#include "nomina.h"
#include <iostream>
#include <cstdlib>
#include <iomanip>
#include <fstream>
#include <cstring>
using std::cout;
using std::setw;

Nomina::Nomina() {
    head = prox = nullptr;
    n = 0;
}

bool Nomina::estaEnNomina( Str n, Str a ) const {
    bool esta = false;

    for ( Nomina * p1 = head; p1 not_eq nullptr; p1 = p1->prox ) {
        if ( a == p1->item.apellido and n == p1->item.nombre ) {
            esta = true;
            break;
        } // if
    } // for

    return esta;
}

int  Nomina::contarRecords() const {
    return n;
}

bool Nomina::estaVacia() const {
    return nullptr == head;
}

void Nomina::entrarRecord( Str no, Str ap, int hs ) {
    Item itm( no, ap, hs );
    Nomina * nuevo = new Nomina;
    nuevo->item = itm;
    Nomina * p1 = head, *p2 = nullptr;

    while ( p1 not_eq nullptr ) {
        if ( ap < p1->item.apellido ) {
            nuevo->prox = p1;      // inserta el nodo entre p2 y p1

            if ( nullptr == p2 ) { // si p2 es nulo...
                head = nuevo;      // ...est en el primer nodo
            } else {               // otros, avanzar
                p2->prox = nuevo;
            } // if-else

            ++n;    // actualiza
            return; // lo insert, nos vamos
        } else {    //  no lo insert, avanzamos
            p2 = p1;
            p1 = p1->prox;
        } // if-else
    } // while

    // si lleg aqu no insert, entonces...
    nuevo->prox = nullptr; // ...nuevo va al final de la lista.

    // chequea de nuevo que p2 no sea nulo
    if ( nullptr == p2 ) {
        head = nuevo;
    } else {
        p2->prox = nuevo;
    } // if-else

    ++n;
}

void Nomina::editarRecord( Str n, Str a, int h ) {
    if ( h <= 0 or not estaEnNomina( n, a ) ) {
        return;
    } // if

    Nomina * p1 = head;

    while ( p1 not_eq nullptr ) {
        if ( n == p1->item.nombre ) { // registro localizado
            p1->item.horas = h;       // valor cambiado
            break;                    // nos vamos
        } else {                      //  no es el registro...
            p1 = p1->prox;            //  ...avanzar
        } // if-else
    } // while
}

void Nomina::borrarRecord( Str no, Str ap ) {
    if ( not estaEnNomina( no, ap ) ) {
        return;
    } // if

    Nomina * p1 = head, *p2 = nullptr;

    while ( p1 not_eq nullptr ) {
        if ( p1->item.nombre == no ) { // lo encontr!

            if ( nullptr == p2 ) {    // 1er. registro?
                head = p1->prox;      // desenlaza
            } else {                  // no lo es?
                p2->prox = p1->prox;  // igual desenlaza
            } // if-else

            --n;                     // actualiza
            delete p1;                // brralo...
            break;                    // ...y vmonos
        } else {                      // no es el registro buscado
            p2 = p1;                  // empareja a p2
            p1 = p1->prox;            // avanza a p1
        } // if-else
    } // while
}

void Nomina::vaciarNomina() {
    Nomina * p1;

    while ( not estaVacia() ) {

        if ( head->prox == nullptr ) { // lleg al ltimo
            delete head;               // brralo
            head = nullptr;            // enlaza y sale
        } else {                       // an hay registros:
            p1 = head;                 // marca la cabeza
            head = p1->prox;           // avanza la cabeza
            delete p1;                 // brralo
        } // if-else
    } // while

    n = 0; // actualiza
}

const int HORAS    =  40;  // a la semana
const int PAGO     = 500;  // pesos
const float BASICO = static_cast<float>( PAGO ) / HORAS;
const float EXTRA  =   1.5 * BASICO;

//  funcin auxiliar: devuelve el salario devengado
float calcSalario( int horas ) {
    float salario;

    if ( horas < HORAS ) {
        salario  = horas * BASICO;
    } else {
        salario  = BASICO;
        salario += ( horas - HORAS ) * EXTRA;
    } // if-else

    return salario;
}

std::ostream & FMT( std::ostream & flujo, Nomina * pHead ) {
    time_t laFecha = time( 0 );  // obteniendo la fecha completa
    Str fecha = ctime( &laFecha ); // la fecha y la hora
    Str nombre;
    int horas;
    float salario;
    float total = 0;

    //  encabezado
    flujo <<
          "CPA S. Garcia - Nomina"  << '\n' <<
          "Fecha: " << fecha << '\n'      << std::left  <<
          setw ( 18 ) << "Obrero"           << std::right <<
          setw ( 7 ) << "Horas"            <<
          setw ( 16 ) << "Salario\n";

    //  la tabla
    for ( Nomina * p1 = pHead; p1 not_eq nullptr; p1 = p1->prox ) {
        if ( p1->item.horas > 0 ) {
            horas = p1->item.horas;
            nombre = p1->item.mostrarInfo();
            salario = calcSalario( horas );
            flujo << std::setfill( '.' ) << std::fixed << std::left <<
                  setw ( 20 ) << nombre        << std::right              <<
                  setw ( 5 ) << horas         << std::setprecision( 2 )  <<
                  setw ( 15 ) << salario << "  _______________\n";
            total += salario;
        } // if
    } // for

    //  el pie
    flujo << std::left  << setw( 30 ) << "Total"
          << std::right << setw( 10 ) << total << "$ Firma\n\n";
    return flujo;
}

#include <iostream>

void Nomina::subirRecords () {
    std::ifstream Lee( "C:/temp/BD/plantilla.txt" ); //  abrir la nominilla

    if ( Lee.fail() ) {
        throw ERR_A;
    } // if

    Str nombre, apellido, horas;

    while ( Lee.good() ) {
        std::getline( Lee, nombre, ',' );       // toma el primer valor
        std::getline( Lee, apellido, ',' );     // toma el segundo
        std::getline( Lee, horas );             // y toma el tercero

        if ( estaEnNomina( nombre, apellido ) ) { // ya est?
            continue;                             // obvialo
        } else {                                  // otros entra el registro
            entrarRecord( nombre, apellido, atoi( horas.c_str() ) );
        } // if-else
    } // while
}
