//  estads.cpp: definiciones para el programa 31
//  Fecha: 7:18 jueves, 2 de mayo de 2019
//  Por S. Galliano.

#include "estads.h"
#include "fileio.h"
#include <cstdlib>

//  funcin de comparacin de dobles, para llamar a Quicksort
int compareDbl ( const void * m, const void * n ) {
    double p = *static_cast<const double *> ( m );
    double q = *static_cast<const double *> ( n );

    if ( p > q ) return  1;
    if ( p < q ) return -1;
    return 0;
}

//	medidas de tendencia central:

// suma de valores
double suma ( const double * ary, int n ) {
    double total = 0;

    for ( int i = 0; i < n; ++i ) {
        total += ary[i];
    } // for

    return total;
}

// media
double promedio ( const double * ary, int n ) {
    return suma ( ary, n ) / n;
}

//  la mediana y la moda requieren de un arreglo ordenado:

// mediana
double mediana ( const double * ary, int n ) {
    //  el arreglo en la tienda libre
    double * data = new double [n];

    //  transferencia a un arreglo no-constante
    for ( int i = 0; i < n; ++i ) {
        data[i] = ary[i];
    } // for

    // ordenamiento
    qsort ( data, n, sizeof ( double ), compareDbl );

    // mediana: C++ cuenta desde cero
    double resp;
    int idx = n / 2;

    if ( 0 == ( n % 2 ) ) {
        resp = 0.5 * ( data[idx - 1] + data[idx] );
    } else {
        resp = data[idx];
    } // if-else

    delete [] data; // liberacin de memoria
    return resp;
}

//  moda
double moda ( const double * ary, int n ) {
    //  el arreglo en la tienda libre
    double * data = new double [n];

    //  transferencia a un arreglo no-constante
    for ( int i = 0; i < n; ++i ) {
        data[i] = ary[i];
    } // for

    // ordenamiento
    qsort ( data, n, sizeof ( double ), compareDbl );

    double unaModa = data[0]; //  el 1er. valor
    double laModa = unaModa;  //  la moda
    int unaRacha = 0;         //  la 1ra. racha
    int laRacha = unaRacha;   //  el tamao de una racha
    int i = 1;                //  el ndice de la secuencia

    while ( i not_eq n ) {
        if ( unaModa not_eq data[i] ) {  //  cambi la unaModa?
            unaModa = data[i];           //  actualizarla

            if ( unaRacha > laRacha ) {  //  cambi la racha?
                laRacha = unaRacha;      //  actualizarla y...
                laModa = data[i - 1]; //  ...actualizar la moda
            } // if-interior

            unaRacha = 0;                //  recomenzar a contar otra racha
        } // if-exterior

        ++unaRacha; //  incrementa la racha
        ++i;        //  incrementa el ndice
    } // while

    if ( 1 == laRacha ) {    //  no hubo rachas mayores que uno?
        return EXIT_FAILURE; //     no hubo unaModa
    } else {                 //  si las hubo?
        return laModa;       //     hubo una moda
    } // if-else
}

#include <cmath>

//	medidas de dispersin:

// varianza muestral
double varianza ( const double * ary, int n ) {
    double total = 0;
    double average = promedio ( ary, n );

    for ( int i = 0; i < n; ++i ) {
        total += pow ( ( ary[i] - average ), 2 );
    } // for

    return ( total / --n );
}

// desviacion tpica
double desviacion ( const double * ary, int n ) {
    return sqrt ( varianza ( ary, n ) );
}

// mximo
double maximo ( const double * ary, int n ) {
    double mayor = ary[0];

    for ( int i = 1; i < n; ++i ) {
        if ( ary[i] > mayor ) {
            mayor = ary[i];
        } // if
    } // for

    return mayor;
}

// mnimo
double minimo     ( const double * ary, int n ) {
    double menor = ary[0];

    for ( int i = 1; i < n; ++i ) {
        if ( ary[i] < menor ) {
            menor = ary[i];
        } // if
    } // for

    return menor;

}

// rango de valores
double rango ( const double * ary, int n ) {
    return maximo ( ary, n ) - minimo ( ary, n );
}

